/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Event application
 * 
 * Copyright (C) 2004, Anthony Minessale
 *
 * Anthony Minessale II <anthm@cylynx.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/manager.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>

static char *tdesc = "Event Application";
static char *app = "Event";
static char *synopsis = "Sends an Event from the dialplan or AGI\n\n"
"Usage: Event(Event=MyEvent|Name=fred|Info=${EXTEN})\n"
"Headers and thier values are up to you.";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int event_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char event_data[1024];
	char tmp[256];
	char event[256];
	char *stack;
	char *var,*val;
	int ok=0;



	if (!data) {
		ast_log(LOG_WARNING, "Event requires at least an Event= arguement\n");
		return -1;
	}
	LOCAL_USER_ADD(u);
	
	stack = (char *) data;
	val=stack;
	while(val) {
		val = strsep(&stack, "|");
		if(!val)
			break;
		var = strsep(&val,"=");
		if(!var)
			break;
		
		if(!strcasecmp(var,"event") && val) {
			strncpy(event,val,sizeof(event));
			ok++;
		}
		else {
			snprintf(tmp,sizeof(tmp),"%s: %s\r\n",var,val);
			strncat(event_data,tmp,sizeof(event_data)-sizeof(tmp));
		}
	}

	if(!ok) {
		ast_log(LOG_WARNING, "Event requires at least an Event= arguement\n");
		return -1;
	}

	manager_event(EVENT_FLAG_CALL,event,"%sExtension: %s\r\nChannel: %s\r\nUniqueid: %s\r\n",event_data,chan->exten,chan->name, chan->uniqueid);
		      


	/* Do our thing here */
	LOCAL_USER_REMOVE(u);
	return res;
}



int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, event_exec, synopsis, tdesc);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
